/**
 * Copyright (C) The MOMENT project, 2004-2006.
 * http://moment.dsic.upv.es
 * 
 * This file is part of the Maude Simple GUI plug-in.
 * Contributed by Abel Gmez, <agomez@dsic.upv.es>.
 * 
 * The Maude Simple GUI plug-in is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * The Maude Simple GUI plug-in is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with the Maude Development Tools plugin, see the file license.txt.
 * If not, write to the Free Software Foundation, Inc., 51 Franklin Street, 
 * Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package es.upv.dsic.issi.moment.maudesimpleGUI;

import org.eclipse.osgi.framework.internal.core.BundleSource;
import org.eclipse.ui.plugin.*;
import org.osgi.framework.BundleContext;

import es.upv.dsic.issi.moment.maudedaemon.core.MaudeDaemonPlugin;
import es.upv.dsic.issi.moment.maudesimpleGUI.core.Maude;

import java.util.*;


/**
 * @author Abel Gmez. agomez@dsic.upv.es
 *
 * The main plugin class to be used in the desktop.
 */


public class MaudesimpleGUIPlugin extends AbstractUIPlugin {
	//The shared instance.
	private static MaudesimpleGUIPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	private Maude maude;
	
	/**
	 * The constructor.
	 */
	public MaudesimpleGUIPlugin() {
		super();
		plugin = this;
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		maude = new Maude();
	}
	
	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		plugin = null;
		resourceBundle = null;
		maude.killMaude();
		MaudeDaemonPlugin.getDefault().deleteMaudeProcess(maude.getMaudeProcess());
		
		maude = null;
	}

	/**
	 * Returns the shared instance.
	 */
	public static MaudesimpleGUIPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = MaudesimpleGUIPlugin.getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		try {
			if (resourceBundle == null)
				resourceBundle = ResourceBundle.getBundle("es.upv.dsic.issi.moment.maudesimpleGUI.messages");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
		return resourceBundle;
	}
	
	public Maude getMaude() {
		return maude;
	}
}
