/**
 * Copyright (C) The MOMENT project, 2004-2006.
 * http://moment.dsic.upv.es
 * 
 * This file is part of the Maude Simple GUI plug-in.
 * Contributed by Abel Gmez, <agomez@dsic.upv.es>.
 * 
 * The Maude Simple GUI plug-in is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * The Maude Simple GUI plug-in is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with the Maude Development Tools plugin, see the file license.txt.
 * If not, write to the Free Software Foundation, Inc., 51 Franklin Street, 
 * Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * Created on 15-feb-2005
 *
 */
package es.upv.dsic.issi.moment.maudesimpleGUI.core;

import org.eclipse.ui.PartInitException;
import org.eclipse.ui.internal.UIPlugin;

import es.upv.dsic.issi.moment.maudedaemon.core.MaudeDaemonPlugin;
import es.upv.dsic.issi.moment.maudedaemon.maude.IMaudeProcessInteract;
import es.upv.dsic.issi.moment.maudesimpleGUI.ui.toconsole.MaudeConsoleView;

/**
 * @author Abel Gmez. agomez@dsic.upv.es
 * 
 */
public class Maude {

	// Maude process
	private static IMaudeProcessInteract maude;
	
	// Display thread
	private ThreadDisplay thDisplayOut;
	private MaudeConsoleView console = null;

	private static final String VIEW_ID = "es.upv.dsic.issi.moment.maudesimpleGUI.ui.views.MaudeOutputView";

	/**
	 * Creates the Maude Process instance.
	 */
	public Maude() {
		maude = MaudeDaemonPlugin.getDefault().getNewMaudeProcessInteract();
		thDisplayOut = new ThreadDisplay();
		thDisplayOut.start();
	}

	/**
	 * Returns the Maude Process instance.
	 */
	public IMaudeProcessInteract getMaudeProcess() {
		return maude;
	}

	public void runMaude() {

//		consoleOut.showConsole();
		console = (MaudeConsoleView) UIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().findView("es.upv.dsic.issi.moment.maudesimpleGUI.ui.toconsole.MaudeConsoleView");
		if (console == null) {
			try {
				UIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().showView("es.upv.dsic.issi.moment.maudesimpleGUI.ui.toconsole.MaudeConsoleView");
				console = (MaudeConsoleView) UIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().findView("es.upv.dsic.issi.moment.maudesimpleGUI.ui.toconsole.MaudeConsoleView");
			} catch (PartInitException e) {
				e.printStackTrace();
			};
		} else {
			console.showConsole();
		}
		
		try {
			maude.execMaude();
		} catch (Exception e) {
		}
		console.setState(MaudeConsoleView.READY);		
	}
	
	public void quitMaude() {
		if (maude != null)
			maude.quitMaude();
		
		//console.clear();
		if (console != null)
			console.setState(MaudeConsoleView.STOPPED);

	}
	
	public void killMaude() {
		if (maude != null)
			maude.killMaude();
		
		//console.clear();
		if (console != null)
			console.setState(MaudeConsoleView.STOPPED);

	}
	
	public boolean isRunning() {
		return maude.isRunning();
	}
	
	public void sendToMaude(String txt) {
		console.setState(MaudeConsoleView.EXECUTING);
		if (console.getShowSent())
			printConsoleOut(txt + "\n");
		maude.sendToMaude(txt);
	}
	
	public void makeCtrlC() {
		maude.makeCtrlC();
	}
	
	public String readOutMaude(int max) {
		return maude.readOutMaudeNoBlock(max);
	}

	public String readErrMaude(int max) {
		return maude.readErrMaudeNoBlock(max);
	}
	private void printConsoleOut(String txt) {
		if (console != null) {
			MaudeConsoleView.getOutputStream().print(txt);
		}
	}
	private void printConsoleErr(String txt) {
		if (console != null) {
			MaudeConsoleView.getErrorStream().print(txt);
		}
	}
	private void printConsolePrompt(String txt) {
		if (console != null) {
			MaudeConsoleView.getPromptStream().print(txt);
		}
	}
	
	private class ThreadDisplay extends Thread {

		public ThreadDisplay() {
			super("Maude Display");
		}
		private static final int maxLen = 4096;
		/**
		 * Run method Actions to do while the thread is active
		 */
		public void run() {
			String txt = "";
			boolean endsWithPrompt;
			boolean running = false;
			while (true) {
				try {
					// Reading of StdErr of the Maude process
					if (maude.isRunning()) {
						running = true;
						try {
							txt = maude.readErrMaudeNoBlock(maxLen);
							while (txt != "") {
								printConsoleErr(txt);
								txt = maude.readErrMaudeNoBlock(maxLen);
							}
						} catch (Exception e) {
							//e.printStackTrace();
						}
					} else if (running) {
						maude.killMaude();
						running = false;
						printConsoleOut("Bye.\n\n");
					}
					// Reading of StdOut of the Maude process
					if (maude.isRunning()) {
						running = true;
						try {
							txt = maude.readOutMaudeNoBlock(maxLen);
							while (txt != "") {
								if (txt.startsWith("Maude> ")) {
									txt = txt.substring(6);
									printConsolePrompt("\nMaude>");
								}
								endsWithPrompt = false;
								if (txt.endsWith("\nMaude> ")) {
									endsWithPrompt = true;
									txt = txt.substring(0,txt.length()-7);
								}
								printConsoleOut(txt);
								if (endsWithPrompt) {
									printConsolePrompt("\nMaude>");
									printConsoleOut(" ");
									
								}
								txt = maude.readOutMaudeNoBlock(maxLen);
							}
						} catch (Exception e) {
							e.printStackTrace();
						}
					} else if (running) {
						maude.killMaude();
						running = false;
						printConsoleOut("Bye.\n\n");
					}
					sleep(10);
				} catch (Exception e) {
				}
			}
		}
	}
}
