/**
 * Copyright (C) The MOMENT project, 2004-2006.
 * http://moment.dsic.upv.es
 * 
 * This file is part of the Maude Simple GUI plug-in.
 * Contributed by Abel Gmez, <agomez@dsic.upv.es>.
 * 
 * The Maude Simple GUI plug-in is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * The Maude Simple GUI plug-in is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with the Maude Development Tools plugin, see the file license.txt.
 * If not, write to the Free Software Foundation, Inc., 51 Franklin Street, 
 * Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */
 




package es.upv.dsic.issi.moment.maudesimpleGUI.ui.editors;

import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.editors.text.TextEditor;

import es.upv.dsic.issi.moment.maudesimpleGUI.MaudesimpleGUIPlugin;
import es.upv.dsic.issi.moment.maudesimpleGUI.Messages;

/**
 * @author Abel Gmez. agomez@dsic.upv.es
 *
 */
public class MaudeEditor extends TextEditor {

	public MaudeEditor()
	{
		super();
		setSourceViewerConfiguration(new MaudeSourceViewerConfig());
	}
	@Override
	protected void createActions() {
		super.createActions();
		Action sendToMaude = new Action(){
			@Override
			public void run() {
				if (!MaudesimpleGUIPlugin.getDefault().getMaude().isRunning()) {
					MessageDialog.openError(null, Messages.getString("MaudeGUI.E_TITLE"),Messages.getString("MaudeGUI.E_NORUNNING")); //$NON-NLS-1$ //$NON-NLS-2$
				} else {
	
					ISelection sel = getSelectionProvider().getSelection();
					Object selection = null;
					if(sel instanceof TextSelection) {
						selection = ((TextSelection)sel).getText();
					}
					if(selection!= null && selection.toString().length() > 0)
						MaudesimpleGUIPlugin.getDefault().getMaude().sendToMaude(selection.toString());
				}
			}

		};
		sendToMaude.setImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_SENDTOMAUDESEL")))));
		sendToMaude.setText(Messages.getString("MaudeGUI.MM_SENDTOMAUDESEL"));
		sendToMaude.setEnabled(true);
		
		setAction("SendToMaudeSel", sendToMaude);

		
	}
	@Override
	protected void editorContextMenuAboutToShow(IMenuManager menu) {
		super.editorContextMenuAboutToShow(menu);
		menu.add(new Separator());
		addAction(menu, "SendToMaudeSel");
	}
}

