/**
 * Copyright (C) The MOMENT project, 2004-2006.
 * http://moment.dsic.upv.es
 * 
 * This file is part of the Maude Simple GUI plug-in.
 * Contributed by Abel Gmez, <agomez@dsic.upv.es>.
 * 
 * The Maude Simple GUI plug-in is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * The Maude Simple GUI plug-in is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with the Maude Development Tools plugin, see the file license.txt.
 * If not, write to the Free Software Foundation, Inc., 51 Franklin Street, 
 * Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */
 




package es.upv.dsic.issi.moment.maudesimpleGUI.ui.editors;

import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.*;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.ide.IDEActionFactory;
import org.eclipse.ui.internal.UIPlugin;
import org.eclipse.ui.part.EditorActionBarContributor;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.texteditor.ITextEditorActionConstants;

import es.upv.dsic.issi.moment.maudesimpleGUI.MaudesimpleGUIPlugin;
import es.upv.dsic.issi.moment.maudesimpleGUI.Messages;
import es.upv.dsic.issi.moment.maudesimpleGUI.ui.toconsole.MaudeConsoleView;

/**
 * @author Abel Gmez. agomez@dsic.upv.es
 *
 * Manages the installation/deinstallation of global actions for multi-page editors.
 * Responsible for the redirection of global actions to the active editor.
 * Multi-page contributor replaces the contributors for the individual editors in the multi-page editor.
 */
public class MaudeEditorContributor extends EditorActionBarContributor {
	private IEditorPart activeEditorPart;
//	private Action actRunMaude;
	private Action actSendMaude;
	private Action actSendSelMaude;
//	private Action actLoopInitMaude;
//	private Action actCtrlCMaude;
//	private Action actQuitMaude;
//	private Action actKillMaude;
	private Action actShowConsole;
	
//	private ToggleCommentAction actComment;

	/**
	 * Creates a multi-page contributor.
	 */
	public MaudeEditorContributor() {
		super();
		createActions();
	}
	/**
	 * Returns the action registed with the given text editor.
	 * @return IAction or null if editor is null.
	 */
	protected IAction getAction(ITextEditor editor, String actionID) {
		return (editor == null ? null : editor.getAction(actionID));
	}
	/* (non-JavaDoc)
	 * Method declared in AbstractMultiPageEditorActionBarContributor.
	 */

	public void setActiveEditor(IEditorPart part) {
		if (activeEditorPart == part)
			return;

		activeEditorPart = part;

		IActionBars actionBars = getActionBars();
		if (actionBars != null) {

			ITextEditor editor = (part instanceof ITextEditor) ? (ITextEditor) part : null;

			actionBars.setGlobalActionHandler(
				ActionFactory.DELETE.getId(),
				getAction(editor, ITextEditorActionConstants.DELETE));
			actionBars.setGlobalActionHandler(
				ActionFactory.UNDO.getId(),
				getAction(editor, ITextEditorActionConstants.UNDO));
			actionBars.setGlobalActionHandler(
				ActionFactory.REDO.getId(),
				getAction(editor, ITextEditorActionConstants.REDO));
			actionBars.setGlobalActionHandler(
				ActionFactory.CUT.getId(),
				getAction(editor, ITextEditorActionConstants.CUT));
			actionBars.setGlobalActionHandler(
				ActionFactory.COPY.getId(),
				getAction(editor, ITextEditorActionConstants.COPY));
			actionBars.setGlobalActionHandler(
				ActionFactory.PASTE.getId(),
				getAction(editor, ITextEditorActionConstants.PASTE));
			actionBars.setGlobalActionHandler(
				ActionFactory.SELECT_ALL.getId(),
				getAction(editor, ITextEditorActionConstants.SELECT_ALL));
			actionBars.setGlobalActionHandler(
				ActionFactory.FIND.getId(),
				getAction(editor, ITextEditorActionConstants.FIND));
			actionBars.setGlobalActionHandler(
				IDEActionFactory.BOOKMARK.getId(),
				getAction(editor, IDEActionFactory.BOOKMARK.getId()));
			actionBars.updateActionBars();
		}
	}
	
	private void createActions() {
//		// Run Maude
//		actRunMaude = new Action() {
//			public void run() {
//				if (MaudesimpleGUIPlugin.getDefault().getMaude().getMaudeProcess().isRunning()) {
//					MessageDialog.openError(null,Messages.getString("MaudeGUI.E_TITLE"),Messages.getString("MaudeGUI.E_ALREADYRUNNIG"));		 //$NON-NLS-1$ //$NON-NLS-2$
//				} else {
//				// We create a new instance of a Maude Process
//					
//					MaudesimpleGUIPlugin.getDefault().getMaude().runMaude();
//				}
//			}
//		};
//		actRunMaude.setText(Messages.getString("MaudeGUI.MM_RUNMAUDE")); //$NON-NLS-1$
//		actRunMaude.setToolTipText(Messages.getString("MaudeGUI.MT_RUNMAUDE")); //$NON-NLS-1$
//		actRunMaude.setAccelerator(SWT.F6);
//		actRunMaude.setImageDescriptor(ImageDescriptor.createFromURL(
//				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_RUNMAUDE"))))); //$NON-NLS-1$
//		actRunMaude.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
//				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_RUNMAUDE"))))); //$NON-NLS-1$
//
		// Sends to Maude
		actSendMaude = new Action() {
			public void run() {

				if (!MaudesimpleGUIPlugin.getDefault().getMaude().isRunning()) {
					MessageDialog.openError(null, Messages.getString("MaudeGUI.E_TITLE"),Messages.getString("MaudeGUI.E_NORUNNING")); //$NON-NLS-1$ //$NON-NLS-2$
				} else {

					TextEditor editor;
					
					editor = (TextEditor) activeEditorPart;
					
					String editorText =
						editor.getDocumentProvider().getDocument(editor.getEditorInput()).get();
					
					MaudesimpleGUIPlugin.getDefault().getMaude().sendToMaude(editorText);
						
				}
			}
		};
		actSendMaude.setText(Messages.getString("MaudeGUI.MM_SENDTOMAUDE")); //$NON-NLS-1$
		actSendMaude.setToolTipText(Messages.getString("MaudeGUI.MT_SENDTOMAUDE")); //$NON-NLS-1$
		actSendMaude.setAccelerator(SWT.F9);
		actSendMaude.setImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_SENDTOMAUDE"))))); //$NON-NLS-1$
		actSendMaude.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_SENDTOMAUDE"))))); //$NON-NLS-1$
		
		// Send selection to Maude
		actSendSelMaude = new Action() {
			public void run() {
				
				if (!MaudesimpleGUIPlugin.getDefault().getMaude().isRunning()) {
					MessageDialog.openError(null, Messages.getString("MaudeGUI.E_TITLE"),Messages.getString("MaudeGUI.E_NORUNNING")); //$NON-NLS-1$ //$NON-NLS-2$
				} else {
	
					TextEditor editor;
					
					editor = (TextEditor) activeEditorPart;
					
					ISelection sel = editor.getSelectionProvider().getSelection();
					Object selection = null;
					if(sel instanceof TextSelection) {
						selection = ((TextSelection)sel).getText();
					}
					if(selection!= null && selection.toString().length() > 0)
						MaudesimpleGUIPlugin.getDefault().getMaude().sendToMaude(selection.toString());
				}
			}
		};
		actSendSelMaude.setText(Messages.getString("MaudeGUI.MM_SENDTOMAUDESEL")); //$NON-NLS-1$
		actSendSelMaude.setToolTipText(Messages.getString("MaudeGUI.MT_SENDTOMAUDESEL")); //$NON-NLS-1$
		actSendSelMaude.setAccelerator(SWT.SHIFT | SWT.F9);
		actSendSelMaude.setImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_SENDTOMAUDESEL"))))); //$NON-NLS-1$
		actSendSelMaude.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_SENDTOMAUDESEL"))))); //$NON-NLS-1$

//		// Loop init
//		actLoopInitMaude = new Action() {
//			public void run() {
//				if (!MaudesimpleGUIPlugin.getDefault().getMaude().isRunning()) {
//					MessageDialog.openError(null, Messages.getString("MaudeGUI.E_TITLE"),Messages.getString("MaudeGUI.E_LOOPINIT_RUN")); //$NON-NLS-1$ //$NON-NLS-2$
//				} else if (!MaudesimpleGUIPlugin.getDefault().getMaude().getMaudeProcess().isFullMaude()) {
//					MessageDialog.openError(null, Messages.getString("MaudeGUI.E_TITLE"),Messages.getString("MaudeGUI.E_LOOPINIT_CORE")); //$NON-NLS-1$ //$NON-NLS-2$
//				} else {
//					MaudesimpleGUIPlugin.getDefault().getMaude().sendToMaude(Messages.getString("MaudeGUI.MAUDELOOPINIT")); //$NON-NLS-1$
//				}
//			}
//		};
//		actLoopInitMaude.setText(Messages.getString("MaudeGUI.MM_LOOPINIT")); //$NON-NLS-1$
//		actLoopInitMaude.setToolTipText(Messages.getString("MaudeGUI.MT_LOOPINIT")); //$NON-NLS-1$
//		actLoopInitMaude.setAccelerator(SWT.F8);
//		actLoopInitMaude.setImageDescriptor(ImageDescriptor.createFromURL(
//				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_LOOPINIT"))))); //$NON-NLS-1$
//		actLoopInitMaude.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
//				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_LOOPINIT"))))); //$NON-NLS-1$
//		// Trace mode
//		actCtrlCMaude = new Action() {
//			public void run() {
//				if (!MaudesimpleGUIPlugin.getDefault().getMaude().isRunning()) {
//					MessageDialog.openError(null, Messages.getString("MaudeGUI.E_TITLE"),Messages.getString("MaudeGUI.E_LOOPINIT_RUN")); //$NON-NLS-1$ //$NON-NLS-2$
//				} else {
//					MaudesimpleGUIPlugin.getDefault().getMaude().makeCtrlC(); 
//				}
//			}
//		};
//		actCtrlCMaude.setText(Messages.getString("MaudeGUI.MM_CTRLCMAUDE")); //$NON-NLS-1$
//		actCtrlCMaude.setToolTipText(Messages.getString("MaudeGUI.MT_CTRLCMAUDE")); //$NON-NLS-1$
//		actCtrlCMaude.setImageDescriptor(ImageDescriptor.createFromURL(
//				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_CTRLCMAUDE"))))); //$NON-NLS-1$
//		actCtrlCMaude.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
//				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_CTRLCMAUDE"))))); //$NON-NLS-1$
//		
//		// Quit Maude
//		actQuitMaude = new Action() {
//			public void run() {
//				MaudesimpleGUIPlugin.getDefault().getMaude().quitMaude();
//			}
//		};
//		actQuitMaude.setText(Messages.getString("MaudeGUI.MM_QUITMAUDE")); //$NON-NLS-1$
//		actQuitMaude.setToolTipText(Messages.getString("MaudeGUI.MT_QUITMAUDE")); //$NON-NLS-1$
//		actQuitMaude.setAccelerator(SWT.F7);
//		actQuitMaude.setImageDescriptor(ImageDescriptor.createFromURL(
//				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_QUITMAUDE"))))); //$NON-NLS-1$
//		actQuitMaude.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
//				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_QUITMAUDE"))))); //$NON-NLS-1$
//
//		// Kill Maude
//		actKillMaude = new Action() {
//			public void run() {
//				MaudesimpleGUIPlugin.getDefault().getMaude().killMaude();
//			}
//		};
//		actKillMaude.setText(Messages.getString("MaudeGUI.MM_KILLMAUDE")); //$NON-NLS-1$
//		actKillMaude.setToolTipText(Messages.getString("MaudeGUI.MT_KILLMAUDE")); //$NON-NLS-1$
//		actKillMaude.setImageDescriptor(ImageDescriptor.createFromURL(
//				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_KILLMAUDE"))))); //$NON-NLS-1$
//		actKillMaude.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
//				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_KILLMAUDE"))))); //$NON-NLS-1$
//
		// Show console
		actShowConsole= new Action() {
			public void run() {
				MaudeConsoleView console = (MaudeConsoleView) UIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().findView("es.upv.dsic.issi.moment.maudesimpleGUI.ui.toconsole.MaudeConsoleView");
				if (console == null) {
					try {
						UIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().showView("es.upv.dsic.issi.moment.maudesimpleGUI.ui.toconsole.MaudeConsoleView");
						console = (MaudeConsoleView) UIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().findView("es.upv.dsic.issi.moment.maudesimpleGUI.ui.toconsole.MaudeConsoleView");
					} catch (PartInitException e) {
					};
				} else {
					console.showConsole();
				}
			}
		};
		actShowConsole.setText(Messages.getString("MaudeGUI.MM_MAUDECONSOLE")); //$NON-NLS-1$
		actShowConsole.setToolTipText(Messages.getString("MaudeGUI.MT_MAUDECONSOLE")); //$NON-NLS-1$
		actShowConsole.setImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_MAUDECONSOLE"))))); //$NON-NLS-1$
		actShowConsole.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_MAUDECONSOLE"))))); //$NON-NLS-1$

		// Comment
//		actComment = new ToggleCommentAction(MaudesimpleGUIPlugin.getDefault().getResourceBundle(),"TOGGLECOMMENT.", (ITextEditor) activeEditorPart);
//		actComment.setEnabled(true);
//		//actComment.setText(Messages.getString("MaudeGUI.MM_KILLMAUDE")); //$NON-NLS-1$
//		//actComment.setToolTipText(Messages.getString("MaudeGUI.MT_KILLMAUDE")); //$NON-NLS-1$
//		actComment.setAccelerator(SWT.CTRL + '/');
//		//actComment.configure(getSourceViewer(), getSourceViewerConfiguration());
//
//		actComment.setImageDescriptor(ImageDescriptor.createFromURL(
//				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_KILLMAUDE"))))); //$NON-NLS-1$
//		actComment.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
//				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_KILLMAUDE"))))); //$NON-NLS-1$
//
	}
	public void contributeToMenu(IMenuManager manager) {
		IMenuManager menu = new MenuManager(Messages.getString("MaudeGUI.EDITOR_MENU_TIT")); //$NON-NLS-1$
		manager.prependToGroup(IWorkbenchActionConstants.MB_ADDITIONS, menu);
//		menu.add(actRunMaude);
//		menu.add(actQuitMaude);
//		menu.add(actKillMaude);
//		menu.add(new Separator());
//		menu.add(actLoopInitMaude);
//		menu.add(new Separator());
//		menu.add(actCtrlCMaude);
//		menu.add(new Separator());
		menu.add(actSendMaude);
		menu.add(actSendSelMaude);
		menu.add(new Separator());
		menu.add(actShowConsole);
//		menu.add(new Separator());
//		menu.add(actComment);
	}
	public void contributeToToolBar(IToolBarManager manager) {
//		manager.add(new Separator());
//		manager.add(actRunMaude);
//		manager.add(actQuitMaude);
//		manager.add(actKillMaude);
//		manager.add(new Separator());
//		manager.add(actLoopInitMaude);
//		manager.add(new Separator());
//		manager.add(actCtrlCMaude);
		manager.add(new Separator());
		manager.add(actSendMaude);
		manager.add(actSendSelMaude);
		manager.add(new Separator());
		manager.add(actShowConsole);
//		manager.add(new Separator());
//		manager.add(actComment);

	}
}
