/**
 * Copyright (C) The MOMENT project, 2004-2006.
 * http://moment.dsic.upv.es
 * 
 * This file is part of the Maude Simple GUI plug-in.
 * Contributed by Abel Gmez, <agomez@dsic.upv.es>.
 * 
 * The Maude Simple GUI plug-in is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * The Maude Simple GUI plug-in is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with the Maude Development Tools plugin, see the file license.txt.
 * If not, write to the Free Software Foundation, Inc., 51 Franklin Street, 
 * Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */
 




package es.upv.dsic.issi.moment.maudesimpleGUI.ui.toconsole;

import java.io.OutputStream;
import java.io.PrintStream;
import java.sql.Time;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.ResourceBundle;
import java.util.SimpleTimeZone;
import java.util.TimeZone;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IFindReplaceTarget;
import org.eclipse.jface.text.ITextListener;
import org.eclipse.jface.text.ITextOperationTarget;
import org.eclipse.jface.text.TextEvent;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleConstants;
import org.eclipse.ui.console.IConsoleView;
import org.eclipse.ui.console.actions.TextViewerAction;
import org.eclipse.ui.internal.UIPlugin;
import org.eclipse.ui.internal.console.ConsoleMessages;
import org.eclipse.ui.internal.console.ConsolePluginImages;
import org.eclipse.ui.internal.console.ConsoleResourceBundleMessages;
import org.eclipse.ui.internal.console.IInternalConsoleConstants;
import org.eclipse.ui.internal.console.IOConsolePartitioner;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.texteditor.FindReplaceAction;
import org.eclipse.ui.texteditor.IWorkbenchActionDefinitionIds;

import es.upv.dsic.issi.moment.maudesimpleGUI.MaudesimpleGUIPlugin;
import es.upv.dsic.issi.moment.maudesimpleGUI.Messages;

/**
 * @author Abel Gmez. agomez@dsic.upv.es
 *
 *
 *	Maude Console View. 
 *
 */

public class MaudeConsoleView extends ViewPart {
    
    // the shared instance of the view
	private static MaudeConsoleView sharedView = null;
	
	// History fields
	private Vector CommandHistory;
	private int	HistoryActive = 0;
	
	// View components
	private Font font;
	private TextViewer scriptTextViewer = null;
	private StyledText scriptTextWidget = null; 
	private Text commandLine = null;
	private Label statusLine = null;
	private Document consoleDoc = new Document();
	private Display display = null;
	
	// Flag to control scroll lock
	private boolean scrollLock;
	// show the commands sent to Maude
	private boolean showSent;
	
	// Output streams
	protected PrintStream outputStream = null;
	protected PrintStream promptStream = null;
	protected PrintStream errorStream = null;

	// Actions
	private Action actRunMaude;
	private Action actLoopInitMaude;
	private Action actCtrlCMaude;
	private Action actQuitMaude;
	private Action actKillMaude;
	private Action actClearConsole;
	private Action actScrollLock;
	private Action actShowSent;
	private TextViewerAction actCopy;
	private TextViewerAction actSelectAll;
	private FindReplaceAction actFind;
	
	// Execution control & timer
	private SimpleDateFormat timeFormat;
	private Date beginTime;
	private boolean executing;
	private int state;
	
	public static final int EXECUTING = 1;
	public static final int STOPPED = 2;
	public static final int READY = 3;
	public static final int ELAPSED = 4;
	

	/**
	 * 
	 * This is the default constructor. It creates the command history vector.
	 * 
	 *
	 */
	
	public MaudeConsoleView() {
	    super();
	    sharedView = this;
	    CommandHistory = new Vector();
	    scrollLock = false;
	    showSent = false;
	    beginTime = null;
	    timeFormat = new SimpleDateFormat("HH:mm:ss.SSS");
	    timeFormat.setTimeZone(new SimpleTimeZone(0,"GMT+00:00"));
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		
		GridLayout gridLayout = new GridLayout();
		
		parent.setLayout(gridLayout);
		display = parent.getDisplay();
		
		statusLine = new Label(parent, SWT.SINGLE | SWT.BOTTOM);
		statusLine.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		font = new Font(null,
				statusLine.getFont().getFontData()[0].getName(),
				statusLine.getFont().getFontData()[0].getHeight(),
				SWT.BOLD);
		
		statusLine.setFont(font);
		
		scriptTextViewer =
			new TextViewer(parent, SWT.MULTI | SWT.V_SCROLL);
		scriptTextWidget = scriptTextViewer.getTextWidget();
		scriptTextWidget.setDoubleClickEnabled(true);
		scriptTextWidget.setFont(JFaceResources.getTextFont());
		scriptTextWidget.setEditable(true);
		scriptTextWidget.setTabs(4);
		scriptTextViewer.setDocument(consoleDoc);
		scriptTextWidget.setForeground(display.getSystemColor(SWT.COLOR_BLUE));
		scriptTextViewer.setEditable(false);
		scriptTextWidget.setLayoutData(new GridData(GridData.FILL_BOTH));
		scriptTextWidget.setWordWrap(true);

    	
		commandLine = new Text(parent, SWT.BORDER | SWT.SINGLE | SWT.BOTTOM);
		commandLine.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		
		initInputReader();
		initOutputStream();
		initPromptStream();
		initErrorStream();
	
	
		createActions();
		contributeToActionBars();
		hookContextMenu();
		
		setState(MaudeConsoleView.STOPPED);
    
	}
	
	// listens to widget and pipes the entered character to given inputWriter
	class WidgetInputListener implements ITextListener, KeyListener {
		boolean ctrlPressed = false;
       public void textChanged(TextEvent te) {}
        public void keyPressed(KeyEvent e) {
    		setState(MaudeConsoleView.READY);
            switch (e.keyCode) {
            	case SWT.CR:
					String command = commandLine.getText();
					if (!command.equals("")) {
						commandLine.setText("");
						CommandHistory.add(command);
		 				HistoryActive = CommandHistory.size();
	
		 				if (MaudesimpleGUIPlugin.getDefault().getMaude().isRunning()) {
		 					if (command.toLowerCase().equals("quit") || command.toLowerCase().equals("eof")) {
		 						MaudesimpleGUIPlugin.getDefault().getMaude().quitMaude();
		 					} else { 
		 						MaudesimpleGUIPlugin.getDefault().getMaude().sendToMaude(command);
		 					}

						}
		 			}
	 				break;

            	case SWT.ARROW_UP:
            		if (HistoryActive > 0) {
            			HistoryActive--;
            			commandLine.setText((String) CommandHistory.elementAt(HistoryActive));
            			commandLine.setSelection(commandLine.getText().length());
            		}
            		e.doit = false;
            		break;

            	case SWT.ARROW_DOWN:
            		if (HistoryActive < CommandHistory.size()-1) {
            			HistoryActive++;
            			commandLine.setText((String) CommandHistory.elementAt(HistoryActive));
            			commandLine.setSelection(commandLine.getText().length());
            		} else {
            			commandLine.setText("");
            		}
            		e.doit = false;
            		break;
            	case SWT.CTRL:
            		ctrlPressed = true;
            		break;
            	case 'f':
            	case 'F':
            		if (ctrlPressed) {
            			ctrlPressed = false;
            			actFind.run();
            		}
            		break;
            }
        }
        public void keyReleased(KeyEvent e) {
        	ctrlPressed = false;
        }
	}
	protected void initInputReader() {
		WidgetInputListener wil = new WidgetInputListener();
		commandLine.addKeyListener(wil);
	}

	
	protected void widgetOutput(final String str, final int len, final int color) {
	    display.asyncExec(new Runnable() {
			public void run() {
				if (isVisible()) {
					int curNdx = scriptTextWidget.getCharCount();
					scriptTextWidget.append(str);
					scriptTextWidget.setStyleRange(new StyleRange(curNdx, len, display.getSystemColor(color), null));
					// ... scroll down automatically
					if (!scrollLock)
						scriptTextWidget.setSelection(scriptTextWidget.getCharCount());
					if (getState() != MaudeConsoleView.ELAPSED)
						setState(MaudeConsoleView.ELAPSED);
				}
			}});
	}

	protected void initOutputStream() {
	    OutputStream tmpStream = new OutputStream() {
			public void write(final int arg0) {
                widgetOutput(new String(new byte[] {(byte) arg0}), 1, SWT.COLOR_BLACK);
			}
			public void write(final byte[] b, final int off, final int len) {
				byte[] tmpBuf = new byte[len];
                System.arraycopy(b, off, tmpBuf, 0, len);
                widgetOutput(new String(tmpBuf), len, SWT.COLOR_BLACK);
			}
		};
		outputStream = new PrintStream(tmpStream);
	}

	protected void initPromptStream() {
	    OutputStream tmpStream = new OutputStream() {
			public void write(final int arg0) {
                widgetOutput(new String(new byte[] {(byte) arg0}), 1, SWT.COLOR_DARK_GREEN);
			}
			public void write(final byte[] b, final int off, final int len) {
				byte[] tmpBuf = new byte[len];
                System.arraycopy(b, off, tmpBuf, 0, len);
                widgetOutput(new String(tmpBuf), len, SWT.COLOR_DARK_GREEN);
			}
		};
		promptStream = new PrintStream(tmpStream);
	}

	protected void initErrorStream() {
	    OutputStream tmpStream = new OutputStream() {
			public void write(final int arg0) {
                widgetOutput(new String(new byte[] {(byte) arg0}), 1, SWT.COLOR_DARK_RED);
			}
			public void write(final byte[] b, final int off, final int len) {
				byte[] tmpBuf = new byte[len];
                System.arraycopy(b, off, tmpBuf, 0, len);
                widgetOutput(new String(tmpBuf), len, SWT.COLOR_DARK_RED);
			}
		};
        errorStream = new PrintStream(tmpStream);
	}
	
	
	public static PrintStream getOutputStream() {
	    if (sharedView != null) {
			return sharedView.outputStream;
	    }
	    return null;
	}

	public static PrintStream getPromptStream() {
	    if (sharedView != null) {
			return sharedView.promptStream;
	    }
	    return null;
	}


	public static PrintStream getErrorStream() {
	    if (sharedView != null) {
			return sharedView.errorStream;
	    }
	    return null;
	}
	
	public void setFocus() {
	    commandLine.setFocus();
	}
	public void showConsole() {
		try {
			PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView("es.upv.dsic.issi.moment.maudesimpleGUI.ui.toconsole.MaudeConsoleView");
		} catch (PartInitException e1) {
			e1.printStackTrace();
		}
	}
	
	public boolean isVisible() {
		return (UIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().findView("es.upv.dsic.issi.moment.maudesimpleGUI.ui.toconsole.MaudeConsoleView") != null);
		
	}
	
	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(new Separator());
		manager.add(actClearConsole);
		manager.add(actScrollLock);
		manager.add(actShowSent);
		manager.add(new Separator());
		manager.add(actRunMaude);
		manager.add(actQuitMaude);
		manager.add(actKillMaude);
		manager.add(new Separator());
		manager.add(actLoopInitMaude);
		manager.add(new Separator());
		manager.add(actCtrlCMaude);
	}
	
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				MaudeConsoleView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(scriptTextViewer.getControl());
		scriptTextViewer.getControl().setMenu(menu);
		scriptTextWidget.setMenu(menu);
		getSite().registerContextMenu(menuMgr, scriptTextViewer);

	}
	
	private void fillContextMenu(IMenuManager manager) {
		
		IDocument doc= scriptTextViewer.getDocument();
		if (doc == null) {
			return;
		}
		manager.add(actCopy);
		manager.add(actSelectAll);
		manager.add(new Separator("FIND")); //$NON-NLS-1$
		manager.add(actFind);
		manager.add(actClearConsole);
		manager.add(new Separator("SCROLLLOCK"));
		manager.add(actScrollLock);
		manager.add(actShowSent);
		
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}
	
	private void createActions() {
		
		//////////////////////////////////////////////////////////////////
		////// Console Actions
		//////////////////////////////////////////////////////////////////
		
		
		// Run Maude
		actRunMaude = new Action() {
			public void run() {
				if (MaudesimpleGUIPlugin.getDefault().getMaude().getMaudeProcess().isRunning()) {
					MessageDialog.openError(null,Messages.getString("MaudeGUI.E_TITLE"),Messages.getString("MaudeGUI.E_ALREADYRUNNIG"));		 //$NON-NLS-1$ //$NON-NLS-2$
				} else {
				// We create a new instance of a Maude Process
					
					MaudesimpleGUIPlugin.getDefault().getMaude().runMaude();
				}
			}
		};
		actRunMaude.setText(Messages.getString("MaudeGUI.MM_RUNMAUDE")); //$NON-NLS-1$
		actRunMaude.setToolTipText(Messages.getString("MaudeGUI.MT_RUNMAUDE")); //$NON-NLS-1$
		actRunMaude.setAccelerator(SWT.F6);
		actRunMaude.setImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_RUNMAUDE"))))); //$NON-NLS-1$
		actRunMaude.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_RUNMAUDE"))))); //$NON-NLS-1$

		// Loop init
		actLoopInitMaude = new Action() {
			public void run() {
				if (!MaudesimpleGUIPlugin.getDefault().getMaude().isRunning()) {
					MessageDialog.openError(null, Messages.getString("MaudeGUI.E_TITLE"),Messages.getString("MaudeGUI.E_LOOPINIT_RUN")); //$NON-NLS-1$ //$NON-NLS-2$
				} else if (!MaudesimpleGUIPlugin.getDefault().getMaude().getMaudeProcess().isFullMaude()) {
					MessageDialog.openError(null, Messages.getString("MaudeGUI.E_TITLE"),Messages.getString("MaudeGUI.E_LOOPINIT_CORE")); //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					MaudesimpleGUIPlugin.getDefault().getMaude().sendToMaude(Messages.getString("MaudeGUI.MAUDELOOPINIT")); //$NON-NLS-1$
				}
			}
		};
		actLoopInitMaude.setText(Messages.getString("MaudeGUI.MM_LOOPINIT")); //$NON-NLS-1$
		actLoopInitMaude.setToolTipText(Messages.getString("MaudeGUI.MT_LOOPINIT")); //$NON-NLS-1$
		actLoopInitMaude.setAccelerator(SWT.F8);
		actLoopInitMaude.setImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_LOOPINIT"))))); //$NON-NLS-1$
		actLoopInitMaude.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_LOOPINIT"))))); //$NON-NLS-1$
		// Trace mode
		actCtrlCMaude = new Action() {
			public void run() {
				if (!MaudesimpleGUIPlugin.getDefault().getMaude().isRunning()) {
					MessageDialog.openError(null, Messages.getString("MaudeGUI.E_TITLE"),Messages.getString("MaudeGUI.E_LOOPINIT_RUN")); //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					MaudesimpleGUIPlugin.getDefault().getMaude().makeCtrlC(); 
				}
			}
		};
		actCtrlCMaude.setText(Messages.getString("MaudeGUI.MM_CTRLCMAUDE")); //$NON-NLS-1$
		actCtrlCMaude.setToolTipText(Messages.getString("MaudeGUI.MT_CTRLCMAUDE")); //$NON-NLS-1$
		actCtrlCMaude.setImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_CTRLCMAUDE"))))); //$NON-NLS-1$
		actCtrlCMaude.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_CTRLCMAUDE"))))); //$NON-NLS-1$
		
		// Quit Maude
		actQuitMaude = new Action() {
			public void run() {
				MaudesimpleGUIPlugin.getDefault().getMaude().quitMaude();
			}
		};
		actQuitMaude.setText(Messages.getString("MaudeGUI.MM_QUITMAUDE")); //$NON-NLS-1$
		actQuitMaude.setToolTipText(Messages.getString("MaudeGUI.MT_QUITMAUDE")); //$NON-NLS-1$
		actQuitMaude.setAccelerator(SWT.F7);
		actQuitMaude.setImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_QUITMAUDE"))))); //$NON-NLS-1$
		actQuitMaude.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_QUITMAUDE"))))); //$NON-NLS-1$

		// Kill Maude
		actKillMaude = new Action() {
			public void run() {
				MaudesimpleGUIPlugin.getDefault().getMaude().killMaude();
			}
		};
		actKillMaude.setText(Messages.getString("MaudeGUI.MM_KILLMAUDE")); //$NON-NLS-1$
		actKillMaude.setToolTipText(Messages.getString("MaudeGUI.MT_KILLMAUDE")); //$NON-NLS-1$
		actKillMaude.setImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_KILLMAUDE"))))); //$NON-NLS-1$
		actKillMaude.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_KILLMAUDE"))))); //$NON-NLS-1$

		// Scroll Lock
		actScrollLock = new Action(null, Action.AS_CHECK_BOX) {
			public void run() {
				scrollLock = !scrollLock;
				actScrollLock.setChecked(scrollLock);
			}
		};
		actScrollLock.setText(Messages.getString("MaudeGUI.MM_SCROLLLOCK")); //$NON-NLS-1$
		actScrollLock.setToolTipText(Messages.getString("MaudeGUI.MT_SCROLLLOCK")); //$NON-NLS-1$
		actScrollLock.setImageDescriptor(ConsolePluginImages.getImageDescriptor(IInternalConsoleConstants.IMG_LCL_LOCK)); //$NON-NLS-1$

		
		// Clear Console
		actClearConsole = new Action() {
			public void run() {
				scriptTextWidget.setText("");
			}
		};
		actClearConsole.setText(Messages.getString("MaudeGUI.MM_CLEARCONSOLE")); //$NON-NLS-1$
		actClearConsole.setToolTipText(Messages.getString("MaudeGUI.MT_CLEARCONSOLE")); //$NON-NLS-1$
		actClearConsole.setImageDescriptor(ConsolePlugin.getImageDescriptor(IConsoleConstants.IMG_LCL_CLEAR)); //$NON-NLS-1$

		// Show Sent
		actShowSent = new Action(null, Action.AS_CHECK_BOX) {
			public void run() {
				showSent = !showSent;
				actShowSent.setChecked(showSent);
			}
		};
		actShowSent.setText(Messages.getString("MaudeGUI.MM_SHOWSENT")); //$NON-NLS-1$
		actShowSent.setToolTipText(Messages.getString("MaudeGUI.MT_SHOWSENT")); //$NON-NLS-1$
		actShowSent.setImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MI_SHOWSENT"))))); //$NON-NLS-1$
		actShowSent.setDisabledImageDescriptor(ImageDescriptor.createFromURL(
				MaudesimpleGUIPlugin.getDefault().find(new Path(Messages.getString("MaudeGUI.MC_SHOWSENT"))))); //$NON-NLS-1$

		
		//////////////////////////////////////////////////////////////////
		////// Popup Actions
		//////////////////////////////////////////////////////////////////

		// Copy
		actCopy = new TextViewerAction(scriptTextViewer, ITextOperationTarget.COPY);
		actCopy.configureAction(ConsoleMessages.TextConsolePage_CopyText, ConsoleMessages.TextConsolePage_CopyDescrip, ConsoleMessages.TextConsolePage_CopyDescrip);
		actCopy.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_TOOL_COPY));
		actCopy.setActionDefinitionId(IWorkbenchActionDefinitionIds.COPY);
		actCopy.setEnabled(true);
		
		// Select All
		actSelectAll = new TextViewerAction(scriptTextViewer, ITextOperationTarget.SELECT_ALL);
		actSelectAll.configureAction(ConsoleMessages.TextConsolePage_SelectAllText, ConsoleMessages.TextConsolePage_SelectAllDescrip, ConsoleMessages.TextConsolePage_SelectAllDescrip);
		actSelectAll.setEnabled(true);
		
		// Find/Replace
		ResourceBundle bundle = ConsoleResourceBundleMessages.getBundle();
		actFind = new FindReplaceAction(bundle, "find_replace_action_", sharedView); //$NON-NLS-1$
		actFind.setAccelerator(SWT.CTRL | 'F');
		actFind.setEnabled(true);	
	}
	

	public Object getAdapter(Class required) {
		if (IFindReplaceTarget.class.equals(required)) {
			return scriptTextViewer.getFindReplaceTarget();
		}
		if (Widget.class.equals(required)) {
			return scriptTextViewer.getTextWidget();
		}
		return null;
	}
	
	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalToolBar(bars.getToolBarManager());
	}

	public boolean getShowSent() {
		return showSent;		
	}
	
	public void setState(int state) {
		switch (state) {
			case MaudeConsoleView.READY:
				display.syncExec(new Runnable() {
					public void run() {
						if (MaudesimpleGUIPlugin.getDefault().getMaude().isRunning())
							statusLine.setText("Ready.");
					}
				});
				break;
			case MaudeConsoleView.EXECUTING:
				if (getState() != MaudeConsoleView.EXECUTING)
					beginTime = new Date();
				
				display.syncExec(new Runnable() {
					public void run() {
						statusLine.setText("Executing...");
					}
				});
				break;
			case MaudeConsoleView.STOPPED:
				display.syncExec(new Runnable() {
					public void run() {
						statusLine.setText("Maude isn't running.");
					}
				});
				break;
			case MaudeConsoleView.ELAPSED:
				display.syncExec(new Runnable() {
					public void run() {
						if (beginTime != null) {
							long elapsed = new Date().getTime() - beginTime.getTime();
							statusLine.setText("Elapsed time: " + timeFormat.format(new Date(elapsed)));
							beginTime = null;
						}
					}
				});
				break;
			default:
				break;
		}
		this.state = state;

	}
	
	public int getState() {
		return state;
	}
	
	public void dispose() {
		super.dispose();
		font.dispose();
	}
}